
#include "transformation.h"
#include "unix.h"
#include "x.h"
#include "xt.h"

/*
 *  Subroutine : Init
 *
 *  Purpose : Initialize the linked list
 *
 */

AnimScript *Init()
{
   AnimScript *anim;

   anim = (AnimScript *) malloc (sizeof( AnimScript ));

   anim->code  = 0;
   anim->move1 = 0.0; anim->move2 = 0.0; anim->move3 = 0.0;
   anim->step1 = 0;   anim->step2 = 0;   anim->step3 = 0;
   anim->next  = NILanim;
   return( anim );
}


/*
 *  Subroutine : read_Script
 *
 *  Purpose : read the transformations from the script and
 *            create the linked list
 *
 */

AnimScript *readTrans( filename )
char *filename;
{
   FILE *ifp, *fopen();
   AnimScript *anim, *prev, *head;
   char ch, *trans;
   float _move;
   int _step;

   if ( (ifp = fopen(filename, "r")) == NULL ) {
      printf( "can't open file %s \n", filename );
      exit(-1);
   }
   else {
      anim = Init();
      head = anim;
      trans = (char *) malloc( 15 );

      while( (ch = fgetc( ifp )) != EOF ) {
         if( ch != 'r' && ch != 's' && ch != 't' ) break;
         ungetc(ch, ifp);

         if( fscanf( ifp, "%s%f%d\n", trans, &_move, &_step ) != 3 ) {
            printf( "could not get transformation %s %f %d\n",
                     trans, _move, _step);
            exit(-1);
         }
         head->code  = whichTrans( trans );
         head->move1 = _move;
         head->step1 = _step;

         if( (head->code >= ONE_AXIS) && (head->code < TWO_AXIS) ) {
            if( fscanf( ifp, "%f%d\n", &_move, &_step ) != 2 ) {
               printf("readScript.c : could not get parameters two\n");
               exit(-1);
            }
            head->move2 = _move;
            head->step2 = _step;
         }
         else if( head->code >= TWO_AXIS ) {
            if( fscanf( ifp, "%f%d\n", &_move, &_step ) != 2 ) {
               printf("readScript.c : could not get parameters three_2\n");
               exit(-1);
            }
            head->move2 = _move;
            head->step2 = _step;

            if( fscanf( ifp, "%f%d\n", &_move, &_step ) != 2 ) {
               printf("readScript.c : could not get parameters three_3\n");
               exit(-1);
            }
            head->move3 = _move;
            head->step3 = _step;
         }
         prev = head;
         head->next = (AnimScript *) malloc (sizeof( AnimScript ));
         head = head->next;
      }
      prev->next = NILanim;
   }
   fclose(ifp);
   return( anim );
}


int whichTrans( trans )
char *trans;
{
        if( !strcmp( trans, "rot_x" ) ) return ROT_X;
   else if( !strcmp( trans, "rot_y" ) ) return ROT_Y;
   else if( !strcmp( trans, "rot_z" ) ) return ROT_Z;
   else if( !strcmp( trans, "rot_xy" ) ) return ROT_XY;
   else if( !strcmp( trans, "rot_yz" ) ) return ROT_YZ;
   else if( !strcmp( trans, "rot_zx" ) ) return ROT_ZX;
   else if( !strcmp( trans, "rot_xyz" ) ) return ROT_XYZ;

   else if( !strcmp( trans, "trn_x" ) ) return TRANS_X;
   else if( !strcmp( trans, "trn_y" ) ) return TRANS_Y;
   else if( !strcmp( trans, "trn_z" ) ) return TRANS_Z;
   else if( !strcmp( trans, "trn_xy" ) ) return TRANS_XY;
   else if( !strcmp( trans, "trn_yz" ) ) return TRANS_YZ;
   else if( !strcmp( trans, "trn_zx" ) ) return TRANS_ZX;
   else if( !strcmp( trans, "trn_xyz" ) ) return TRANS_XYZ;

   else if( !strcmp( trans, "scl_x" ) ) return SCALE_X;
   else if( !strcmp( trans, "scl_y" ) ) return SCALE_Y;
   else if( !strcmp( trans, "scl_z" ) ) return SCALE_Z;
   else if( !strcmp( trans, "scl_xy" ) ) return SCALE_XY;
   else if( !strcmp( trans, "scl_yz" ) ) return SCALE_YZ;
   else if( !strcmp( trans, "scl_zx" ) ) return SCALE_ZX;
   else if( !strcmp( trans, "scl_xyz" ) ) return SCALE_XYZ;

   else return UNKNOWN;
}
