
#include "x.h"
#include "xt.h"
#include "globals.h"
#include "defs.h"
#include "externs.h"
#include <math.h>
#ifdef XPM
#include "X11/xpm.h"
#endif

Pixmap mark;

typedef struct _Text {
    char *direction;
    char *point;
} Text, *TextPtr;
TextPtr text;

#define xlogo11_width 11
#define xlogo11_height 11
static char xlogo11_bits[] = {
 0x0f, 0x04, 0x0f, 0x02, 0x1e, 0x01, 0x3c, 0x01, 0xb8, 0x00, 0x58, 0x00,
 0xe8, 0x00, 0xe4, 0x01, 0xc4, 0x03, 0xc2, 0x03, 0x81, 0x07 };

#define lightTypeMENUITEMS 4
char * lightTypeMENUEntry[] = {
/* "Ambient", "Directional", "Positional", "Spot", */
   "Ambient", "Vector", "Point", "Spot"
};

#define lightMENUITEMS 8
static String lightMENUEntry[] = {
   "all on", "blank",
/* "Ambient", "Directional", "Positional", "Spot", */
   "Ambient", "Vector", "Point", "Spot",
   "blank", "all off"
};

#define shadeMENUITEMS 4
char * shadeMENUEntry[] = {
   "None", "Color", "Dot Product", "Normal"
};

#define reflectMENUITEMS 4
char * reflectMENUEntry[] = {
   "None", "Ambient", "Diffuse", "Specular"
};

#define depthCueMENUITEMS 2
char * depthCueMENUEntry[] = {
   "suppressed", "allowed"
};

#define hlhsrMENUITEMS 2
char * hlhsrMENUEntry[] = {
   "off", "on"
};

#define ASCIITEXTATTRIBUTES        \
    XtNeditType, XawtextEdit,      \
    XtNtype, XawAsciiString,       \
    XtNuseStringInPlace, True,     \
    XtNresizable, True,            \
    XtNresize, XawtextResizeWidth, \
    XtNright, XtChainRight,


Widget lightEntry[lightMENUITEMS],
       shadeEntry[shadeMENUITEMS],
       reflectEntry[reflectMENUITEMS],
       lightTypeEntry[lightTypeMENUITEMS],

       lightColor, lightDirection, lightPoint,
       lightAttenuation1, lightAttenuation1Scroll, lightAttenuation1Label,
       lightAttenuation2, lightAttenuation2Scroll, lightAttenuation2Label,
       lightConcentration, lightConcentrationScroll,lightConcentrationLabel,
       lightSpreadAngle, lightSpreadAngleScroll, lightSpreadAngleLabel,

       lightColorLabel, lightDirectionText, lightPointText,
                        lightDirectionTextOk, lightPointTextOk;

#ifdef XPM
void errorMessageXPM(ErrorStatus, tag)
int ErrorStatus;
char *tag;
{
    char *error = NULL;
    char *warning = NULL;

    switch (ErrorStatus) {
    case XpmSuccess:
        return;
    case XpmColorError:
        warning = "Could not parse or alloc requested color";
        break;
    case XpmOpenFailed:
        error = "Cannot open file";
        break;
    case XpmFileInvalid:
        error = "Invalid XPM file";
        break;
    case XpmNoMemory:
        error = "Not enough memory";
        break;
    case XpmColorFailed:
        error = "Failed to parse or alloc some color";
        break;
    }

    if (warning)
        printf("%s Xpm Warning: %s.\n", tag, warning);

    if (error) {
        printf("%s Xpm Error: %s.\n", tag, error);
/*      exit( -1 ); */
    }
}
#endif


void
create_gui( depth, vendor )
int depth;
char *vendor;
{
    int i, status;
    float topOfThumb;
    XtArgVal *l_topOfThumb;
    String string = (String) malloc(40);
    Dimension width, height;
    extern PEXLightEntry lights[4];

    static XtCallbackRec callbacks[] =
    {
        {(XtCallbackProc) NULL, (XtPointer) NULL},
        {(XtCallbackProc) NULL, (XtPointer) NULL},
    };

    extern Widget toplevel;
    Widget form,
           lightForm,
           lightTypeLabel, lightTypeButton, lightTypeMenu; 

    Widget lightButton, shadeButton, reflectButton;
    Widget lightMenu,   shadeMenu,   reflectMenu;

#ifdef XPM
    Pixmap backPixmap, pix;
    XpmAttributes attributes;

    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) ) {
        attributes.valuemask = XpmReturnPixels;
        status = XpmReadFileToPixmap( XtDisplay(toplevel),
            DefaultRootWindow(XtDisplay(toplevel)),
            "pixmaps/background.xpm", &backPixmap, &pix, &attributes );
        errorMessageXPM( status, "Background" );
    }
#endif

    /* Create the bitmap for marking selected items. */
    mark = XCreateBitmapFromData(
        XtDisplay( toplevel ),
        RootWindowOfScreen( XtScreen( toplevel ) ),
        xlogo11_bits, xlogo11_width, xlogo11_height );

    text = (TextPtr) malloc( sizeof (Text) );

    /* Create the widgets. */
    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
        NULL );

    lightButton = XtVaCreateManagedWidget( "lightButton", menuButtonWidgetClass,
        form,
        XtNlabel, "Lights",
        XtNmenuName, "lightMenu",
        NULL );
    lightMenu = XtCreatePopupShell( "lightMenu", simpleMenuWidgetClass,
        lightButton,
        NULL, 0 );
    XtVaSetValues( lightMenu,
        XtNvisual, DefaultVisual( dpy, DefaultScreen( dpy ) ),
        XtNdepth, DefaultDepth( dpy, DefaultScreen( dpy ) ),
        XtNcolormap, DefaultColormap( dpy, DefaultScreen( dpy ) ),
        NULL );

    for( i=0; i < lightMENUITEMS; i++ ) {
        String item = lightMENUEntry[i];

        if (i == 1)
            lightEntry[i] = XtVaCreateManagedWidget( item, smeLineObjectClass,
                lightMenu,
                XtNlineWidth, 5,
                NULL );
        else if (i == 6)
            lightEntry[i] = XtVaCreateManagedWidget( item, smeLineObjectClass,
                lightMenu,
                XtNlineWidth, 5,
                NULL );
        else {
            extern int light_switches[4];
            lightEntry[i] = XtCreateManagedWidget( item, smeBSBObjectClass,
                lightMenu, NULL, 0 );
            if( (i!=0) && (i!=7) && (light_switches[i-2] != 0) )
                XtVaSetValues( lightEntry[i], XtNleftBitmap, mark, NULL );
        }
        XtAddCallback( lightEntry[i], XtNcallback, set_lights, (XtPointer) i);
    }

    shadeButton = XtVaCreateManagedWidget( "shadeButton", menuButtonWidgetClass,
        form,
        XtNlabel, "Shading",
        XtNfromHoriz, lightButton,
        XtNmenuName, "shadeMenu",
        NULL );
    shadeMenu = XtCreatePopupShell( "shadeMenu", simpleMenuWidgetClass,
        shadeButton, NULL, ZERO );
    XtVaSetValues( shadeMenu,
        XtNvisual, DefaultVisual( dpy, DefaultScreen( dpy ) ),
        XtNdepth, DefaultDepth( dpy, DefaultScreen( dpy ) ),
        XtNcolormap, DefaultColormap( dpy, DefaultScreen( dpy ) ),
        NULL );

    for( i=0; i < shadeMENUITEMS; i++ ) {
        char * item = shadeMENUEntry[i];

        shadeEntry[i] = XtVaCreateManagedWidget( item, smeBSBObjectClass,
            shadeMenu, NULL );
        if( i+1 == cur_shading )
            XtVaSetValues( shadeEntry[i], XtNleftBitmap, mark, NULL );
        XtAddCallback( shadeEntry[i], XtNcallback, set_shading, (XtPointer) i);
    }

    reflectButton = XtVaCreateManagedWidget( "reflectButton",
        menuButtonWidgetClass, form,
        XtNlabel, "Reflectance",
        XtNfromHoriz, shadeButton,
        XtNmenuName, "reflectMenu",
        NULL );
    reflectMenu = XtCreatePopupShell( "reflectMenu", simpleMenuWidgetClass,
        reflectButton, NULL, ZERO );
    XtVaSetValues( reflectMenu,
        XtNvisual, DefaultVisual( dpy, DefaultScreen( dpy ) ),
        XtNdepth, DefaultDepth( dpy, DefaultScreen( dpy ) ),
        XtNcolormap, DefaultColormap( dpy, DefaultScreen( dpy ) ),
        NULL );

    for( i=0; i < reflectMENUITEMS; i++ ) {
        char * item = reflectMENUEntry[i];

        reflectEntry[i] = XtVaCreateManagedWidget( item, smeBSBObjectClass,
            reflectMenu, NULL );
        if( i+1 == cur_refl )
            XtVaSetValues( reflectEntry[i], XtNleftBitmap, mark, NULL );
        XtAddCallback( reflectEntry[i], XtNcallback, set_reflectance, (XtPointer) i);
    }

    canvas = XtVaCreateManagedWidget( "canvas", formWidgetClass, form,
        XtNwidth, 210,
        XtNheight, 210,
        XtNfromVert, lightButton,
        NULL );
    XtAddEventHandler( canvas, ExposureMask, FALSE, expose, NULL );

    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    lightForm = XtVaCreateManagedWidget( "lightForm", formWidgetClass,
        form,
        XtNfromVert, lightButton,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    lightForm = XtVaCreateManagedWidget( "lightForm", formWidgetClass,
        form,
        XtNfromVert, lightButton,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
        NULL );

    lightTypeLabel = XtVaCreateManagedWidget( "lightType", labelWidgetClass,
        lightForm,
        XtNborderWidth, 0,
        XtNlabel, "Current Light:",
        NULL );

    lightTypeButton = XtVaCreateManagedWidget( "lightTypeButton",
        menuButtonWidgetClass, lightForm,
        XtNfromHoriz, lightTypeLabel,
        XtNlabel, "Spot",
        XtNwidth, 70,
        XtNmenuName, "lightTypeMenu",
        NULL );
    lightTypeMenu = XtVaCreatePopupShell( "lightTypeMenu",
        simpleMenuWidgetClass,
        lightTypeButton,
        NULL );
    XtVaSetValues( lightTypeMenu,
        XtNvisual, DefaultVisual( dpy, DefaultScreen( dpy ) ),
        XtNdepth, DefaultDepth( dpy, DefaultScreen( dpy ) ),
        XtNcolormap, DefaultColormap( dpy, DefaultScreen( dpy ) ),
        NULL );

    for( i=0; i < lightTypeMENUITEMS; i++ ) {
        char * item = lightTypeMENUEntry[i];

        lightTypeEntry[i] = XtVaCreateManagedWidget( item,
            smeBSBObjectClass,
            lightTypeMenu, NULL );

        if( i == 3 )
        XtVaSetValues( lightTypeEntry[i], XtNleftBitmap, mark, NULL );

        XtAddCallback( lightTypeEntry[i], XtNcallback, select_cur_light,
                       (XtPointer) i);
    }

    lightColor = XtVaCreateManagedWidget( "lightColor", commandWidgetClass,
        lightForm,
        XtNlabel, "Color:",
        XtNfromVert, lightTypeLabel,
        XtNvertDistance, 10,
        XtNwidth, 100,
        NULL );
    XtAddCallback( lightColor, XtNcallback, editColor, NULL);

    sprintf( string, "R: %.2f G: %.2f B: %.2f", lights[SPOT].color.value.rgb.red, lights[SPOT].color.value.rgb.green, lights[SPOT].color.value.rgb.blue );
    lightColorLabel = XtVaCreateManagedWidget( "lightColorLabel",
        labelWidgetClass,
        lightForm,
        XtNfromVert, lightTypeLabel,
        XtNfromHoriz, lightColor,
        XtNlabel, string,
        XtNwidth, 194,
        XtNvertDistance, 10,
        NULL );

    lightDirection = XtVaCreateManagedWidget( "lightDirection",
        commandWidgetClass,
        lightForm,
        XtNlabel, "Direction:",
        XtNfromVert, lightColor,
        XtNwidth, 100,
        NULL );

    string = (String) malloc( 30 );
    sprintf( string, "x: %.2f y: %.2f z: %.2f", lights[SPOT].direction.x, lights[SPOT].direction.y, lights[SPOT].direction.z );
    text->direction = XtNewString( string );
    lightDirectionText = XtVaCreateManagedWidget( "lightDirectionText",
        asciiTextWidgetClass,
        lightForm,
        ASCIITEXTATTRIBUTES
        XtNstring, text->direction,
        XtNlength, 30,
        XtNwidth, 11*strlen(text->direction),
        XtNinsertPosition, strlen(text->direction),
        XtNfromVert, lightColor,
        XtNfromHoriz, lightDirection,
        XtNwidth, 194,
        NULL );

    lightDirectionTextOk = XtVaCreateManagedWidget( "lightDirectionTextOk",
        commandWidgetClass,
        lightForm,
        XtNfromVert, lightColor,
        XtNfromHoriz, lightDirectionText,
        XtNlabel, "Ok",
        NULL );
    XtAddCallback( lightDirectionTextOk, XtNcallback, set_dir_pnt, (XtPointer)DIRECTION );

    lightPoint = XtVaCreateManagedWidget( "lightPoint",
        commandWidgetClass,
        lightForm,
        XtNlabel, "Point:",
        XtNfromVert, lightDirection,
        XtNwidth, 100,
        NULL );

    string = (String) malloc( 30 );
    sprintf( string, "x: %.2f y: %.2f z: %.2f", lights[SPOT].point.x, lights[SPOT].point.y, lights[SPOT].point.z );
    text->point = XtNewString( string );
    lightPointText = XtVaCreateManagedWidget( "lightPointText",
        asciiTextWidgetClass,
        lightForm,
        ASCIITEXTATTRIBUTES
        XtNstring, text->point,
        XtNlength, 30,
        XtNwidth, 11*strlen(text->point),
        XtNinsertPosition, strlen(text->point),
        XtNfromVert, lightDirection,
        XtNfromHoriz, lightPoint,
        XtNwidth, 194,
        NULL );

    lightPointTextOk = XtVaCreateManagedWidget( "lightPointTextOk",
        commandWidgetClass,
        lightForm,
        XtNfromVert, lightDirectionTextOk,
        XtNfromHoriz, lightPointText,
        XtNlabel, "Ok",
        NULL );
    XtAddCallback( lightPointTextOk, XtNcallback, set_dir_pnt, (XtPointer)POINT );

    lightAttenuation1 = XtVaCreateManagedWidget( "lightAttenuation1",
        labelWidgetClass,
        lightForm,
        XtNlabel, "Attenuation 1:",
        XtNwidth, 100,
        XtNfromVert, lightPoint,
        NULL );
    XtVaGetValues( lightAttenuation1, XtNheight, &height, NULL );

    topOfThumb = lights[SPOT].attenuation1;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    lightAttenuation1Scroll = XtVaCreateManagedWidget( "Attenuation1Scroll",
        scrollbarWidgetClass,
        lightForm,
        XtNfromVert, lightPoint,
        XtNfromHoriz, lightAttenuation1,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( lightAttenuation1Scroll, XtNjumpProc, set_light_source, (XtPointer)ATTENUATION_C1 );

    string = (String) malloc( 30 );
    sprintf( string, "%.2f", lights[SPOT].attenuation1 );
    lightAttenuation1Label = XtVaCreateManagedWidget( "Attenuation1Label",
        labelWidgetClass,
        lightForm,
        XtNfromVert, lightPoint,
        XtNfromHoriz, lightAttenuation1Scroll,
        XtNlabel, string,
        XtNheight, height,
        NULL );
    XtVaGetValues( lightAttenuation1Label, XtNwidth, &width, NULL );

    lightAttenuation2 = XtVaCreateManagedWidget( "lightAttenuation2",
        labelWidgetClass,
        lightForm,
        XtNlabel, "Attenuation 2:",
        XtNwidth, 100,
        XtNfromVert, lightAttenuation1,
        NULL );

    topOfThumb = lights[SPOT].attenuation2;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    lightAttenuation2Scroll = XtVaCreateManagedWidget( "Attenuation2Scroll",
        scrollbarWidgetClass,
        lightForm,
        XtNfromVert, lightAttenuation1,
        XtNfromHoriz, lightAttenuation2,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( lightAttenuation2Scroll, XtNjumpProc, set_light_source, (XtPointer)ATTENUATION_C2 );

    string = (String) malloc( 30 );
    sprintf( string, "%.2f", lights[SPOT].attenuation2 );
    lightAttenuation2Label = XtVaCreateManagedWidget( "Attenuation2Label",
        labelWidgetClass,
        lightForm,
        XtNfromVert, lightAttenuation1,
        XtNfromHoriz, lightAttenuation2Scroll,
        XtNlabel, string,
        XtNheight, height,
        NULL );

    lightConcentration = XtVaCreateManagedWidget( "lightConcentration",
        labelWidgetClass,
        lightForm,
        XtNlabel, "Concentration:",
        XtNwidth, 100,
        XtNfromVert, lightAttenuation2,
        NULL );

    topOfThumb = lights[SPOT].concentration / 200.;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    lightConcentrationScroll=XtVaCreateManagedWidget("lightConcentrationScroll",
        scrollbarWidgetClass,
        lightForm,
        XtNfromVert, lightAttenuation2,
        XtNfromHoriz, lightConcentration,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( lightConcentrationScroll, XtNjumpProc, set_light_source, (XtPointer)CONCENTRATION );

    string = (String) malloc( 30 );
    sprintf( string, "%.0f", lights[SPOT].concentration );
    lightConcentrationLabel = XtVaCreateManagedWidget("lightConcentrationLabel",
        labelWidgetClass,
        lightForm,
        XtNfromVert, lightAttenuation2,
        XtNfromHoriz, lightConcentrationScroll,
        XtNlabel, string,
        XtNwidth, width,
        XtNheight, height,
        NULL );

    lightSpreadAngle = XtVaCreateManagedWidget( "lightSpreadAngle",
        labelWidgetClass,
        lightForm,
        XtNlabel, "SpreadAngle:",
        XtNwidth, 100,
        XtNfromVert, lightConcentration,
        NULL );

    topOfThumb = lights[SPOT].spread_angle * M_PI/2;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    lightSpreadAngleScroll = XtVaCreateManagedWidget( "lightSpreadAngleScroll",
        scrollbarWidgetClass,
        lightForm,
        XtNfromVert, lightConcentration,
        XtNfromHoriz, lightSpreadAngle,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( lightSpreadAngleScroll, XtNjumpProc, set_light_source, (XtPointer)SPREAD_ANGLE );

#define RADDEG( _a )  ( ( float )( _a ) * 180.0 / 3.1415926 )

    string = (String) malloc( 30 );
    sprintf( string, "%.0f", RADDEG(lights[SPOT].spread_angle) );
    lightSpreadAngleLabel = XtVaCreateManagedWidget( "lightSpreadAngleLabel",
        labelWidgetClass,
        lightForm,
        XtNfromVert, lightConcentration,
        XtNfromHoriz, lightSpreadAngleScroll,
        XtNlabel, string,
        XtNwidth, width,
        XtNheight, height,
        NULL );

    callbacks[0].callback = (XtCallbackProc) finalQuit;
    callbacks[0].closure  = (XtPointer) NULL;
    XtVaCreateManagedWidget( "quit", commandWidgetClass, form,
        XtNcallback, callbacks,
        XtNlabel, "Quit",
        XtNwidth, 100,
        XtNfromVert, canvas,
        NULL);

    XtRealizeWidget( toplevel );
}

void
finalQuit( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    extern XtAppContext app;
    extern void printLight();

    printLight();
    XtDestroyApplicationContext( app );
    (void) exit (0);
}
